# ============================================================================
# Copyright 2015. Quantum Corporation. All Rights Reserved.
# StorNext is either a trademark or registered trademark of
# Quantum Corporation in the US and/or other countries.
# ============================================================================
# This is a library of common subroutines to be used by fslattusrpt script
# ============================================================================

package CommonLatRpt;

use strict;
use warnings;
use POSIX;
use English;                                # For $PID & $PROGRAM_NAME
use File::Basename;                         # For basename
use Alib;                                   # For SourceShl
use LWP::Simple;                            # For curl calls
use SNDB;                                   # For database access

use Exporter;
our @ISA = 'Exporter';
our @EXPORT = qw($programName);

# Global variables
my $BASEPATH      = ".";        # The Software base path
my $ADIC_PROFILE  = "/usr/adic/.profile";   # The main profile
my $debugPrint    = undef;                  # Should we print debug info?
my $debugLogOn    = undef;                  # Debug Logging on/off
my $logfh         = undef;                  # The logging file handle
our $programName  = basename($PROGRAM_NAME);# The name of the running program
my $pid           = $PID;                   # The pid of the program

# The month_name is used for logging below
my @month_name = qw( Jan Feb Mar Apr May Jun Jul Aug Sept Oct Nov Dec );

# ============================================================================
# Sets a flag to indicate that we want to print debugging information
# ============================================================================
sub debugPrinting($)
{
    my($mode) = @_;
    $debugPrint = $mode;

}

# ============================================================================
# This is used to print a line
#
# IN: The Message to print
# ============================================================================
sub println($)
{
    my ($msg) = @_;

    print "$msg\n" ;
}

# ============================================================================
# This is used to print a message and then exit
#
# IN: The message to log
# IN (optional): The exit value. If not specified will be 1
# ============================================================================
sub printAndExit($;$)
{
    my ($msg, $exitValue) = @_;

    println($msg);

    $exitValue = 1 unless defined $exitValue;

    exit($exitValue);
}

# ============================================================================
# Called to exit with a good value
# ============================================================================
sub exitGood()
{
    exit(0);
}

# ============================================================================
# Called to exit with a bad value
# ============================================================================
sub exitBad()
{
    exit(1);
}

# ============================================================================
# run the the specified command and return the status code of the command
#
# IN: The command to run
# IN (optional): Do not log an error as an error
# ============================================================================
sub justRunIt($;$)
{
    my($cmd, $dontLogErr) = @_;

    if ( $debugPrint ) {
        println("Running $cmd");
    } 

    my $out = `$cmd`;
    my $rc = $?;

    if ( $rc == 0 ) {
        logdebug("cmd=$cmd, rc=$rc, output=[$out]", 1);
    } else {
        $rc >>= 8;

        if ($dontLogErr) {
            logdebug("cmd=$cmd, rc=$rc, output=[$out]", 1);
        } else {
            logerr("ERROR: cmd=$cmd failed, rc=$rc, output=[$out]", 1);
        }
    }

    return $rc;
}

# ============================================================================
# Used for logging later to get the timestamp
#
# RTN: the date/timestamp
# ============================================================================
sub datestamp()
{
    my ($sec, $min, $hr, $mday, $mon) = localtime();

    return sprintf("%s %02d %02d:%02d:%02d",
                    $month_name[$mon], $mday, $hr, $min, $sec);

}

# ============================================================================
# log a message (Used by other log methods)
#
# IN: The log level string
# IN: The message to log
# IN: (optional) Do we print the message also?
# ============================================================================
sub logmsg($@)
{
    my ($level, $msg, $doPrint) = @_;
    if ( $logfh ) {
        printf $logfh datestamp . " $programName" . "[$pid] %-4s: $msg\n", $level;
    }

    println($msg) if $doPrint;
}

# ============================================================================
# Log by level.
#
# IN: message to log
# IN: (optional) print it
# ============================================================================
#sub logerr($;$) { logmsg("ERR", @_); }
#sub logwarn($;$) { logmsg("WARN", @_); }
#sub logoper($;$) { logmsg("OPER", @_); }
sub logdebug($;$)
{
    #return unless $debugLogOn;

    # Turn off 2nd arg if $debugPrint is 0 or undef.
    my @ary = ( $_[0], $debugPrint && $_[1] );
    logmsg("DBG", @ary );
}


# ============================================================================
# Return a date string representing the timestamp.
# If timestamp is undefined, use current time.
#
# IN: timestamp
# RTN: Date string, in the format of: 07-27-15 14:28:39 
# ============================================================================
sub getDateString(;$)
{
    my ($tstamp) = @_;
    $tstamp = time unless defined $tstamp;

    my $dateFmt = "%d-%b-%y %T";
    my $dateStr = strftime( $dateFmt, localtime($tstamp) );
    return $dateStr;
}



# ============================================================================
# Called to determine if TSM is running or not.
#
# RTN: 0 if not up, 1 if up
# ============================================================================
sub isTsmUp()
{
    my $rc = 0;

    if ( justRunIt("TSM_control status", 1) == 0 ) {
        $rc = 1;
    } else {
        println("TSM is not active");
    }

    return $rc;
}

# ============================================================================
# Initialize items:
# logs, configuration, webservices, and email variables.
# This will EXIT if there's a failure of any sort
#
# ============================================================================
sub initialize()
{
    my ($debugEnvVar, $execName) = @_;
    $execName = $programName if ( !defined $execName );

    # Source the parent profile
    SourceShl($ADIC_PROFILE);

    # Check if TSM is up or NOT
    # Have to 'exitGood' to avoid cron issues
    if ( isTsmUp() == 0 ) {
        println("TSM is not active, but is " .
                "required to be running for this program.");
        exitGood();
    }
}

1;      # Required to use
